#ifndef IGNORANCE_LEVEL_H
#define IGNORANCE_LEVEL_H

#include "ignorance_rule.h"


//	describes an ignorance level
//
//	index is an index for keeping track
//	of levels' relative positions
//	the idea is for 0 to be the "default" level,
//	>0 to be "better" levels, and <0 to be "worse"
//
//	allow_passthrough flags whether a user
//	is allowed to be "passed through" this level
//	to the next consecutive level, as in:
//	if someone on my friends list does something
//	that flags 12 of my rules, do I send them 
//	straight to /dev/null, or just 
//	bump them down a level?
//
//	name - user's name for the level
//
//	denizens - list of users in this level
//	I may get rid of this in favor of a more global
//	userlist
//
//	rules - list of rules for this level
typedef struct ignorance_level{
	GString		*name;
	GList		*denizens;
	GPtrArray	*rules;
} ignorance_level;

ignorance_level*	ignorance_level_new();
void				ignorance_level_free(ignorance_level *il);	//DEEP destruction!
void				ignorance_level_free_g(gpointer il,gpointer user_data);	//DEEP destruction!

//adds the given rule to the given level and returns success/failure
gboolean	ignorance_level_add_rule(ignorance_level *level,ignorance_rule *rule);

//returns the named rule from the given level, or NULL
ignorance_rule*	ignorance_level_get_rule(ignorance_level *level,const GString *rulename);

//removes the named rule from the given level, and returns success/failure
gboolean	ignorance_level_remove_rule(ignorance_level *level,const GString *rulename);

//adds a given user to a given level, and returns success/failure
gboolean	ignorance_level_add_denizen(ignorance_level *level,const GString *username);

//adds a given user to a given level, and returns success/failure
//adds without searching first, for use with bulk adds (e.g. zinc import)
gboolean	ignorance_level_add_denizen_fast(ignorance_level *level,const GString *username);

//checks a given level for a given user
gboolean	ignorance_level_has_denizen(ignorance_level *level,const GString *username);

#ifdef HAVE_REGEX_H
gboolean	ignorance_level_has_denizen_regex(ignorance_level	*level,const gchar *regex, GList	**denizens);
#endif

//removes a given user from a given level, and returns success/failure
gboolean	ignorance_level_remove_denizen(ignorance_level *level,const GString *username);



//	Determines whether a string violates one of 
//	the rules defined in an ignorance level
//
//	il is the ignorance level
//	text is the possibly offending string
//	flags are the rule flags to match
//	returns score
gint	ignorance_level_rulecheck(ignorance_level *il,const GString *username,const GString *text, gint flags,GList **violations);

//generates a level from its textual description and returns it, or NULL
ignorance_level*	ignorance_level_read(const gchar *lvltext);

// Writes an ignorance_level to file
//
// level is the level to be output
// f is the file to be written
//
// returns success/failure
gboolean	ignorance_level_write(ignorance_level *level,FILE *f);

#endif
