#include "ignorance_rule.h"
#include <string.h>
#include <strings.h>



gboolean	assign_rule_token(ignorance_rule *rule,const char *tokentxt);

 ignorance_rule*	ignorance_rule_new(){
	ignorance_rule *ir=(ignorance_rule*)g_malloc(sizeof(ignorance_rule));

	ir->name=g_string_new("");
	ir->type=IGNORANCE_RULE_SIMPLETEXT;
	ir->score=0;
	ir->flags=0;
	ir->enabled=TRUE;
	ir->value=NULL;
	ir->message=NULL;
	ir->sound=NULL;
	ir->command=NULL;

	return ir;
}//ignorance_rule_new



ignorance_rule*	ignorance_rule_newp(const GString *name,
					gint type,
					const gchar *value,
					gint score, 
					gint flags,
					gboolean enabled,
					const gchar *message,
					const gchar *sound,
					const gchar *command){
	ignorance_rule *ir=(ignorance_rule*)g_malloc(sizeof(ignorance_rule));

	ir->name=g_string_new(name->str);
	if(ignorance_rule_has_type(type)){
		ir->type=type;
	} else {
		ir->type=IGNORANCE_RULE_INVALID;
	}//if
	ir->value=g_strdup(value);
	ir->score=score;
	ir->flags=flags;
	ir->enabled=enabled;
	ir->message=g_strdup(message);
	ir->sound=g_strdup(sound);
	ir->command=g_strdup(command);

	return ir;
}//ignorance_rule_newp


void	ignorance_rule_free(ignorance_rule *ir){
	g_string_free(ir->name,TRUE);
	g_free(ir->value);
	g_free(ir->message);
	g_free(ir->sound);
	g_free(ir->command);
	g_free(ir);
}//ignorance_rule_free


void	ignorance_rule_free_g(gpointer ir,gpointer user_data){
	ignorance_rule_free((ignorance_rule*)ir);
}//ignorance_rule_free_g


gboolean	ignorance_rule_has_type(gint type){
	if((type>=IGNORANCE_RULE_MINVALID) || (type<=IGNORANCE_RULE_INVALID))
		return FALSE;
	return TRUE;
}//ignorance_rule_has_type


gint	ignorance_rule_rulecheck(ignorance_rule *rule,const GString *text, gint flags){
	if((flags & rule->flags) && rule->enabled){//match flag types
		switch(rule->type){
			case IGNORANCE_RULE_SIMPLETEXT:
				return simple_text_rulecheck(rule,text);
#ifdef HAVE_REGEX_H
			case IGNORANCE_RULE_REGEX:
				return regex_rulecheck(rule,text);
#endif
			default:
				return 0;
		}//switch
	}//if

	return 0;
}//ignorance_rule_rulecheck


gint	simple_text_rulecheck(ignorance_rule *rule,const GString *text){
	const gchar *rulevalue=(const gchar*)(rule->value);
	 
	if(NULL!=g_strstr_len(text->str,text->len,rulevalue)){
		return rule->score;
	}//if

	return 0;
}//simple_text_rulecheck


#ifdef HAVE_REGEX_H
gint	regex_rulecheck(ignorance_rule *rule, const GString *text){
	regex_t	reg;
	gint rv=0;
	
	if(regcomp(&reg,(const gchar*)rule->value,REG_EXTENDED | REG_NOSUB)){
		gaim_debug(GAIM_DEBUG_ERROR, "ignorance", "Error parsing regex %s\n",(const gchar*)(rule->value));
	}else if(!regexec(&reg,text->str,1,NULL,0)){
		rv=rule->score;
	}//if
		
	regfree(&reg);
	return rv;
}//regex_rulecheck
#endif


ignorance_rule*	ignorance_rule_read_old(const gchar *ruletext){
	gchar	*tokptr=strchr((gchar*)ruletext,' '),
			**tokens=NULL;
	ignorance_rule *rule=ignorance_rule_new();
	int i=0;

	if(!tokptr){
		ignorance_rule_free(rule);
		return NULL;
	}

	tokens=g_strsplit(ruletext," ",INT_MAX);

	for(i=0;tokens[i];++i){
		assign_rule_token(rule,tokens[i]);
	}//iterate tokens

	if(rule->score > 9 || rule->score < -9){
		rule->score=IGNORANCE_FLAG_IGNORE;
	}else{
		rule->score=IGNORANCE_FLAG_FILTER;
	}//switch to new scoring system

	g_strfreev(tokens);

	return rule;
}//ignorance_level_read


ignorance_rule*	ignorance_rule_read(const gchar *ruletext){
	gchar	*tokptr=strchr((gchar*)ruletext,'\n'),
			**tokens;
	ignorance_rule *rule=ignorance_rule_new();
	int i=0;


	if(!tokptr){
		//old version of config file
		ignorance_rule_free(rule);
		return ignorance_rule_read_old(ruletext);
	}

	tokens=g_strsplit(ruletext,"\n",INT_MAX);

	for(i=0;tokens[i];++i){
		assign_rule_token(rule,tokens[i]);
	}//iterate tokens

	g_strfreev(tokens);

	return rule;
}//ignorance_level_read




// Parses out a token of the form tokenname="value" and assigns it to a rulename
// 
// rule is the rule to be updated
// tokentxt is the token string
// true returned if token is valid and successfully added to the rule
//
//level name1="value1" name2="value2" ...
gboolean	assign_rule_token(ignorance_rule *rule,const gchar *tokentxt){
	gchar	*name=(gchar*)tokentxt,
		*value=NULL;
	gboolean rv=TRUE;
	gint	cursor=0;

		value=strchr(tokentxt,'=');
		if(value){	
			(*value)='\0';
			++value;

			//strip "value" quotes
			if('"'==(*value)){
				++value;
				cursor=strlen(value)-1;
				if('"'==value[cursor]){
					value[cursor]='\0';
				}//if
			}//if

			//HACK!
			if(!strncasecmp(name,"name",BUFSIZ)){
				g_string_assign(rule->name,value);
			}else if(!strncasecmp(name,"type",BUFSIZ)){
				rule->type=atoi(value);
			}else if(!strncasecmp(name,"value",BUFSIZ)){
				rule->value=(gchar*)g_malloc((strlen(value)+1)*sizeof(gchar));
				strncpy(rule->value,value,strlen(value)+1);
			}else if(!strncasecmp(name,"score",BUFSIZ)){
				rule->score=atoi(value);
			}else if(!strncasecmp(name,"flags",BUFSIZ)){
				rule->flags=atoi(value);
			}else if(!strncasecmp(name,"enabled",BUFSIZ)){
				rule->enabled=(gboolean)atoi(value);
			}else if(!strncasecmp(name,"message",BUFSIZ)){
				rule->message=(gchar*)g_malloc((strlen(value)+1)*sizeof(gchar));
				strncpy(rule->message,value,strlen(value)+1);
			}else if(!strncasecmp(name,"command",BUFSIZ)){
				rule->command=(gchar*)g_malloc((strlen(value)+1)*sizeof(gchar));
				strncpy(rule->command,value,strlen(value)+1);
			}else if(!strncasecmp(name,"sound",BUFSIZ)){
				rule->sound=(gchar*)g_malloc((strlen(value)+1)*sizeof(gchar));
				strncpy(rule->sound,value,strlen(value)+1);
			}else{
				rv=FALSE;
			}//long, ugly switch
		}else{
			rv=FALSE;
		}//value

		return rv;
}//assign_rule_token


gboolean	ignorance_rule_write(ignorance_rule *rule,FILE *f){
	fprintf(f,"rule\nname=\"%s\"\ntype=\"%d\"\nscore=\"%d\"\nvalue=\"%s\"\nflags=\"%d\"\nenabled=\"%d\"\nmessage=\"%s\"\ncommand=\"%s\"\nsound=\"%s\"\n/rule\n",
			rule->name->str,rule->type,rule->score,(gchar*)(rule->value),rule->flags,rule->enabled,rule->message,rule->command,rule->sound);
			
	return TRUE;
}//ignorance_rule_write
